;Copyright Alexander Meshcheryakov (Self-Perfection), 2009
;Contact me: alexander.s.m@gmail.com
;distributed under BSD license

;Used assumptions:
;   1) Screen width bigger then height

include "macros.inc"

background_cl = 0x000000
foreground_cl = 0xFFFFFF

delay = 5

; debug = 1

MEOS_APP_START

CODE
;Preinit. Randomize start counter
    mcall 3
    mov [counter], eax

draw_window:
    ;Query screen size
    mov     eax,14
    mcall
    mov     [y_max], ax
    shr     eax, 16
    mov     [x_max], ax

    ; start redraw  (without calling 12 proc our window overwrites window that above it)
    mcall   12, 1
    xor eax,eax
    movzx ebx, [x_max]
    movzx ecx, [y_max]
    mov edx,0x01000000
    mcall           ;Define window
    mov  eax,13
    xor  edx,edx
    ;It appears that window is one px smaller then rect with same size constraints
    inc ebx
    inc ecx
    mcall           ;Fill window rect

    shr ebx, 1
    mov [x_center], bx
    shr ecx, 1
    mov [radius], cx

    if defined debug
        ;   Display number of steps
        mov eax, 47
        mov ebx, 0+0+65536*7
        mov ecx, dword [counter]
        mov edx, 30*65536+40    ;coords
        mov esi, 0x80000000 + foreground_cl
        int 0x40
    end if
    
    fninit
    fild word [half_line_number]
    fldpi
    fdiv st, st1    ;Now st0 contains angle step of line start points
    
    movzx ecx, [half_line_number]
    shl ecx, 1
    fldz

next_line:
    push ecx
    fld st
    fld st
    
    ;Calculate line start points
    fsincos
    fimul [radius]
    fiadd [x_center]
    fistp [start_x]
    fchs
    fimul [radius]
    fiadd [radius]
    fistp [start_y]
    
    ;Calculate line end points
    fimul [counter]
    fidiv [divider]
    fsincos
    fimul [radius]
    fiadd [x_center]
    fistp [end_x]
    fchs
    fimul [radius]
    fiadd [radius]
    fistp [end_y]
    
    ;Draw line
    mov ebx, dword [start_x]
    mov ecx, dword [start_y]
    mov edx, foreground_cl
    mcall 38

    fadd st0, st1
    pop ecx
    dec ecx
    jnz next_line
    
    inc dword [counter]

    mcall   12, 2       ; end redraw

wait_event:
    ;mcall   10
    mcall 23, delay
    test eax,eax
    jz  draw_window     ; Delay passed
    dec eax
    jz  draw_window
    dec eax
    jnz  exit
; key pressed, read it and ignore
    mcall   2
    cmp eax, 0x1B00        ;Test for Escape key press
    jnz wait_event

; button pressed; we have only one button, close
; also seems to handle Alt+F4
exit:
    mcall   -1


DATA
counter             dd 0000
divider             dw 1000
half_line_number    dw 180

UDATA
x_max            dw ?     ; ࠧ ࠭
y_max            dw ?

radius          dw ?
x_center        dw ?

start_x         dw ?
end_x         dw ?

start_y         dw ?
end_y         dw ?

MEOS_APP_END